#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#define IPV6_JOIN_ANYCAST	27
#define IPV6_LEAVE_ANYCAST	28
#define ipv6_areq		ipv6_mreq
#define ipv6ar_anyaddr		ipv6mr_multiaddr
#define ipv6ar_interface	ipv6mr_interface

int main(int argc, char *argv[])
{
	struct addrinfo hints, *res;
	struct ipv6_areq areq;
	int s;
	int gai;
	char *anycast;

	if (argc < 2) {
		fprintf(stderr,
			"Usage: %s anycast-address\n",
			argv[0]);
		exit(1);
	}

	anycast = argv[1];

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_INET6;
	hints.ai_socktype = SOCK_DGRAM;

	gai = getaddrinfo(anycast, NULL, &hints, &res);
	if (gai) {
		fprintf(stderr,
			"getaddrinfo(): %s\n", 
			anycast);
		exit(1);
	}

	s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	if (s < 0) {
		perror("socket");
		exit(1);
	}

#if 0
	if (bind(s, res->ai_addr, res->ai_addrlen) < 0) {
		perror("bind");
		exit(1);
	}
#endif

	memset(&areq, 0, sizeof(areq));
	memcpy(&areq.ipv6ar_anyaddr, 
	       &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr,
	       sizeof(areq.ipv6ar_anyaddr));
#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
	areq.ipv6ar_interface = ((struct sockaddr_in6 *)res->ai_addr)->sin6_scope_id;
#endif

	if (setsockopt(s, IPPROTO_IPV6, IPV6_JOIN_ANYCAST, &areq, sizeof(areq)) < 0) {
		perror("setsockopt(IPV6_JOIN_ANYCAST)");
		exit(1);
	}

	getchar();

}

